<?php

namespace Drupal\media_twig_tools\EventSubscribers;

use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\media_twig_tools\Services\ImageBuilder;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Intercept a 404 response and check if it is due to an ungenerated Image.
 *
 * @package Drupal\media_twig_tools\EventSubscribers
 */
class ImageBuilderEventSubscriber implements EventSubscriberInterface, TrustedCallbackInterface {

  /**
   * ImageBuilder.
   *
   * @var \Drupal\media_twig_tools\Services\ImageBuilder
   */
  protected $imageBuilder;

  /**
   * ImageBuilderEventSubscriber constructor.
   *
   * @param \Drupal\media_twig_tools\Services\ImageBuilder $imageBuilder
   *   The ImageBuilder service.
   */
  public function __construct(ImageBuilder $imageBuilder) {
    $this->imageBuilder = $imageBuilder;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::RESPONSE => 'onResponse',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['onResponse'];
  }

  /**
   * On Kernel response.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $responseEvent
   *   The response event.
   */
  public function onResponse(ResponseEvent $responseEvent) {
    $pathInfo = $responseEvent->getRequest()->getPathInfo();
    if ($this->imageBuilder->isImageBuilderRoute($pathInfo)
      && $responseEvent->getResponse()->getStatusCode() === Response::HTTP_NOT_FOUND) {
      try {
        $generator = $this->imageBuilder->generateImage($pathInfo);

        if ($generator->fileExists()) {
          // Image exists. Redirect to the same URL that is no more 404.
          $redirectResponse = new RedirectResponse($responseEvent->getRequest()
            ->getPathInfo());
          $redirectResponse->send();
        }
      }
      catch (\Exception $e) {
        // Mute exception...
      }
    }
  }

}
